#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['sec_cik_mapper']

package_data = \
{'': ['*']}

install_requires = \
['requests', 'pandas', 'typing_extensions']

extras_require = \
{'doc': ['sphinx', 'sphinx-autodoc-typehints'],
 'test': ['pytest', 'pytest-cov']}

setup(name='sec-cik-mapper',
      version='2.1.0',
      description='Generate mappings between SEC identifiers using Python',
      author=None,
      author_email='Jad Chaar <jad.chaar@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
